# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.cache import patch_vary_headers

from .utils import get_donottrack


class DoNotTrackMiddleware(object):
    """
    Middleware that inspects the ``HTTP_DNT`` header and provides information
    for your app to act appropriately.

    """

    def process_request(self, request):
        """
        Inspects the request for the ``HTTP_DNT`` header and sets a
        ``donottrack`` attribute on the request object appropriately.

        Doing this here, rather than in a context processor, allows for your
        views to also take advantage of this logic.

        """
        request.donottrack = get_donottrack(request)

        return None

    def process_response(self, request, response):
        """
        Adds a vary header for ``DNT``, allowing for cache control based on the
        ``HTTP_DNT`` header.

        """
        patch_vary_headers(response, ('DNT',))

        return response



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.http import HttpRequest, HttpResponse
from django.test import TestCase

from ..context_processors import donottrack
from ..middleware import DoNotTrackMiddleware
from ..utils import get_donottrack


class GetDoNotTrackTestCase(TestCase):

    def setUp(self):
        self.request = HttpRequest()

    def test_http_dnt_header_not_present(self):
        self.assertFalse(get_donottrack(self.request))

    def test_http_dnt_header_is_1(self):
        self.request.META['HTTP_DNT'] = '1'
        self.assertTrue(get_donottrack(self.request))

    def test_http_dnt_header_is_not_1(self):
        self.request.META['HTTP_DNT'] = 1
        self.assertFalse(get_donottrack(self.request))

        self.request.META['HTTP_DNT'] = '0'
        self.assertFalse(get_donottrack(self.request))

        self.request.META['HTTP_DNT'] = 'wat'
        self.assertFalse(get_donottrack(self.request))


class DoNotTrackMiddlewareTestCase(TestCase):

    def test_process_request_sets_donottrack_attr(self):
        request = HttpRequest()
        result = DoNotTrackMiddleware().process_request(request)

        # Verify that ``None`` is returned, so Django will continue to process
        # the request.
        self.assertIsNone(result)

        # Now verify the side effect of the previous process_request call.
        self.assertTrue(hasattr(request, 'donottrack'))

    def test_process_response_adds_dnt_vary(self):
        response = DoNotTrackMiddleware().process_response(
                HttpRequest(), HttpResponse())
        # TODO: Use regex to be sure we match the right thing. This one fails
        # because we cannot check for start of string in the look-behind.
        # (look-behind requires fixed-width pattern.) Not sure how to get it
        # matching proplerly. ('DNT', 'WAT,DNT' 'DNT,WAT', 'WIDNT', 'DNTDOIT')
        # self.assertRegexpMatches(response['VARY'], r'(?<=^|,|\s)DNT(?=,|$)')
        self.assertIn('DNT', response['VARY'])


class DoNotTrackContextProcessorTestCase(TestCase):

    def test_donottrack_added_to_context(self):
        request = HttpRequest()
        DoNotTrackMiddleware().process_request(request)
        self.assertIn('donottrack', donottrack(request))

    def test_missing_middleware_raises_exception(self):
        with self.assertRaises(AttributeError):
            donottrack(HttpRequest())



# -*- coding: utf-8 -*-

from __future__ import unicode_literals


def get_donottrack(request):
    """
    Returns ``True`` if ``HTTP_DNT`` header is ``'1'``, ``False`` otherwise.

    """
    return request.META.get('HTTP_DNT') == '1'



# -*- coding: utf-8 -*-

"""
Module required for Django to recognize ``donottrack`` as an app.

And that is necessary for tests to run. This module serves no other purpose.

"""

from __future__ import unicode_literals



# -*- coding: utf-8 -*-

"""
Django utilities for honoring the `Do Not Track HTTP header`_.

`Mathieu Leplatre`_ posted an article |Do not forget DNT|_. I really
appreciated the post, and wanted to implement the ideas in my projects. Being
lazy, I only wanted to do the work once, so I took his ideas and packaged them
up with some tests and docs.

Included is middleware for detecting ``HTTP_DNT`` and passing its information
on to both views and templates via the ``request`` object. The middleware also
adds a vary header for cache control.

This package requires Python 2.7 or newer and Django 1.4 or later.

Refer to the documentation_ for further information.

.. _Do Not Track HTTP header: http://donottrack.us/
.. _Mathieu Leplatre : https://twitter.com/leplatrem
.. |Do not forget DNT| replace:: `Django: Do not forget Do Not Track`
.. _Do not forget DNT:
    http://blog.mathieu-leplatre.info/django-do-not-forget-do-not-track.html
.. _documentation: http://django-donottrack.readthedocs.org/

"""

from __future__ import unicode_literals


__version__ = '0.1'



# -*- coding: utf-8 -*-

from __future__ import unicode_literals


def donottrack(request):
    """
    Adds ``donottrack`` to the context, which is ``True`` if the ``HTTP_DNT``
    header is ``'1'``, ``False`` otherwise.

    This context processor requires installtion of the
    ``donottrack.middleware.DoNotTrackMiddleware``.

    Note that use of this context processor is not strictly necessary. (Though
    it is quite convenient.) If you are using the
    ``django.core.context_processors.request`` context processor, you have
    access to ``{{ request.donottrack }}``.

    """
    # We could just use the ``donottrack.utils.get_donottrack`` function rather
    # than rely on the middlware, but we want to require the middleware so that
    # the VARY header is properly patched to account for DNT.
    try:
        return {'donottrack': request.donottrack}
    except AttributeError:
        raise AttributeError("'WSGIRequest' object has no attribute 'donottrack'"
            " - 'donottrack.middleware.DoNotTrackMiddleware' must be in your"
            " MIDDLEWARE_CLASSES")



# -*- coding: utf-8 -*-

import os

from distutils.core import setup


here = os.path.dirname(__file__)


def get_long_desc():
    return open(os.path.join(here, 'README.rst')).read()


# Function borrowed from carljm.
def get_version():
    fh = open(os.path.join(here, 'donottrack', '__init__.py'))
    try:
        for line in fh.readlines():
            if line.startswith('__version__ ='):
                return line.split('=')[1].strip().strip("'")
    finally:
        fh.close()


setup(
    name='django-donottrack',
    version=get_version(),
    description='Django utilities for honoring the Do Not Track HTTP header.',
    url='https://github.com/benspaulding/django-donottrack/',
    author='Ben Spaulding',
    author_email='ben@benspaulding.us',
    license='BSD',
    download_url='https://github.com/benspaulding/django-donottrack/tarball/v{0}'.format(get_version()),
    long_description=get_long_desc(),
    packages=[
        'donottrack',
        'donottrack.tests',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Browsers',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
)



# -*- coding: utf-8 -*-

"""
Settings for running app tests when not part of another project.

"""

from __future__ import unicode_literals


# Requred by Django, though we don't actually use the database.
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = (
    'donottrack',
)

# Again, required by Django.
SECRET_KEY = 'super-secret!'
